﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using DoctorEye;

namespace Forth.Ics.DoctorEye.SDK.Tools.CS
{
    internal partial class CreateSessionsControl : UserControl
    {

        private global::DoctorEye.Plugins.Plugin _Plugin;

        private global::DoctorEye.Session MyFirstSession;
        private global::DoctorEye.DicomDataSet MyFirstDataSet;

        private global::DoctorEye.Session MySecondSession;
        private global::DoctorEye.DicomDataSet MySecondDataSet;

        private global::DoctorEye.Session MyOutputSession;
        private global::DoctorEye.DicomDataSet MyOutputDataSet;

        private String[] Session1Files = new String[] { };
        private String[] Session2Files = new String[] { };
        private String OutputFileTemplate = System.String.Empty;
        
        internal global::DoctorEye.Plugins.Plugin Plugin
        {
            get { return _Plugin; }
            set { _Plugin = value; }
        }

        public CreateSessionsControl()
        {
            InitializeComponent();
        }

        private void Button1_Click(object sender, EventArgs e)
        {
            if (this.OFD.ShowDialog(this) == DialogResult.OK)
            {
                this.Label1.Text = String.Format("{0} selected file(s)", this.OFD.FileNames.Length);
                this.Session1Files = (from item in this.OFD.FileNames orderby item select item).ToArray();
            }
        }

        private void Button2_Click(object sender, EventArgs e)
        {
            if (this.OFD.ShowDialog(this) == DialogResult.OK)
            {
                this.Label2.Text = String.Format("{0} selected file(s)", this.OFD.FileNames.Length);
                this.Session2Files = (from item in this.OFD.FileNames orderby item select item).ToArray();
            }
        }

        private void Button4_Click(object sender, EventArgs e)
        {
            if (this.SFD.ShowDialog(this) == DialogResult.OK)
            {
                this.Label3.Text = System.IO.Path.GetFileName(this.SFD.FileName);
                this.OutputFileTemplate = this.SFD.FileName;
            }
        }

        private void Button3_Click(object sender, EventArgs e)
        {
            if (this.Session1Files.Length == 0 || this.Session2Files.Length == 0 || System.String.IsNullOrEmpty(this.OutputFileTemplate))
            {
                MessageBox.Show("Please fill all the required information and try again", "Incomplete information", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            this.CreateSessions();

            this.ExecuteCalculations();

            // After all the loading and calculations is finished, show the output session.
            this.MyOutputSession.Visible = true;

        }

        private void CreateSessions()
        {
            // Create the first session
            this.MyFirstSession = global::DoctorEye.Application.AddSession();

            // Add a new dataset to the session we created to store our images.
            this.MyFirstDataSet = this.MyFirstSession.AddDataset();

            using (DicomDataSet TheDataSet = global::DoctorEye.DicomDataSet.CreateNew())
            {
                foreach (string item in this.Session1Files)
                {
                    TheDataSet.AddImage(item).Load(false);
                }

                var SortedList = from item in TheDataSet.Images
                                 orderby item.InstanceNumber, item.FullName
                                 select item.FullName;

                // Add the images and load them
                foreach (string item in SortedList)
                {
                    this.MyFirstDataSet.AddImage(item).Load(true);
                }

            } 

            // After all the loading is done, show the session
            this.MyFirstSession.Visible = true;

            // Create the second session
            this.MySecondSession = global::DoctorEye.Application.AddSession();

            // Add a new dataset to the session we created to store our images.
            this.MySecondDataSet = this.MySecondSession.AddDataset();

            using (DicomDataSet TheDataSet = global::DoctorEye.DicomDataSet.CreateNew())
            {
                foreach (string item in this.Session2Files)
                {
                    TheDataSet.AddImage(item).Load(false);
                }

                var SortedList = from item in TheDataSet.Images
                                 orderby item.InstanceNumber, item.FullName
                                 select item.FullName;

                // Add the images and load them
                foreach (string item in SortedList)
                {
                    this.MySecondDataSet.AddImage(item).Load(true);
                }

            } 

            // After all the loading is done, show the session
            this.MySecondSession.Visible = true;

            // Create the output session
            this.MyOutputSession = global::DoctorEye.Application.AddSession();

            // Add a new dataset to the session we created to store our images.
            this.MyOutputDataSet = this.MyOutputSession.AddDataset();

            string FilePath = Path.GetDirectoryName(this.OutputFileTemplate);
            string FilenameWithoutExt = Path.GetFileNameWithoutExtension(this.OutputFileTemplate);
            string FilenameExt = Path.GetExtension(this.OutputFileTemplate);
            for (int i = 0; i <= Math.Min(this.Session1Files.Length, this.Session2Files.Length) - 1; i++)
            {
                Size FirstSize = this.MyFirstDataSet.Images[i].Size;
                Size SecondSize = this.MySecondDataSet.Images[i].Size;

                // We keep the maximum size of the two images for the result image
                Size OutputSize = new Size(Math.Max(FirstSize.Width, SecondSize.Width), Math.Max(FirstSize.Height, SecondSize.Height));
                string OutputFilename = Path.Combine(FilePath, Path.ChangeExtension(string.Format("{0}_{1}", FilenameWithoutExt, i + 1), FilenameExt));

                if (this.RadioButtonCopyFromFirst.Checked)
                {
                    // Copy the image to the output directory
                    System.IO.File.Copy(this.MyFirstDataSet.Images[i].FullName, OutputFilename, true);

                    // Add the Dicom Image to the list
                    this.MyOutputDataSet.AddImage(OutputFilename).Load(false);
                }
                else
                {
                    // Create the Dicom Image and add it to the list
                    this.MyOutputDataSet.AddImage(OutputFilename, OutputSize).Load(true);
                }
            }
        }

        private void ExecuteCalculations()
        {
            for (int i = 0; i <= Math.Min(this.MyFirstDataSet.Images.Count, this.MySecondDataSet.Images.Count) - 1; i++)
            {
                DicomImage MyFirstImage = this.MyFirstDataSet.Images[i];
                DicomImage MySecondImage = this.MySecondDataSet.Images[i];
                DicomImage MyOutputImage = this.MyOutputDataSet.Images[i];

                // We get a list of unique annotation labels.
                var AnnotationLabels = ((from ann1 in MyFirstImage.Annotations
                                        select ann1.Label).Union(
                                        from ann2 in MySecondImage.Annotations
                                        select ann2.Label)).Distinct();

                foreach (String label in AnnotationLabels)
                {
                    var LabelToSearch = label;
                    var Ann1 = (from item in MyFirstImage.Annotations
                                where (item.Label == LabelToSearch)
                                select item).FirstOrDefault();

                    var Ann2 = (from item in MySecondImage.Annotations
                                where (item.Label == LabelToSearch)
                                select item).FirstOrDefault();

                    // We need annotations that exist on both images.
                    if (Ann1 != null && Ann2 != null) 
                    {
                        // We copy the first annotation to the output image
                        Annotation OutputAnn1 = MyOutputImage.AddAnnotation(string.Format("{0} (First Image)", LabelToSearch));
                        OutputAnn1.SuspendLayout();
                        OutputAnn1.Color = Color.FromArgb(50, Color.Red.R, Color.Red.G, Color.Red.B);
                        OutputAnn1.Selection = Ann1.Selection;
                        OutputAnn1.ResumeLayout();

                        // We copy the second annotation to the output image
                        Annotation OutputAnn2 = MyOutputImage.AddAnnotation(string.Format("{0} (Second Image)", LabelToSearch));
                        OutputAnn2.SuspendLayout();
                        OutputAnn2.Color = Color.FromArgb(50, Color.Green.R, Color.Green.G, Color.Green.B);
                        OutputAnn2.Selection = Ann2.Selection;
                        OutputAnn2.ResumeLayout();

                        // We create a third annotation with the common selection to the output image
                        Annotation OutputAnn3 = MyOutputImage.AddAnnotation(string.Format("{0} (Common)", LabelToSearch));
                        OutputAnn3.SuspendLayout();
                        OutputAnn3.Color = Color.FromArgb(50, Color.Blue.R, Color.Blue.G, Color.Blue.B);

                        // We create a fourth annotation with the subtraction selection to the output image
                        Annotation OutputAnn4 = MyOutputImage.AddAnnotation(string.Format("{0} (Subtraction)", LabelToSearch));
                        OutputAnn4.SuspendLayout();
                        OutputAnn4.Color = Color.FromArgb(50, Color.Cyan.R, Color.Cyan.G, Color.Cyan.B);

                        // The common selection points must be in the common "areas" of the two images.
                        int MinX = Math.Min(MyFirstImage.Size.Width, MySecondImage.Size.Width);
                        int MinY = Math.Min(MyFirstImage.Size.Height, MySecondImage.Size.Height);

                        // We select the common and subtraction points of the two images.
                        for (int x = 0; x <= MinX - 1; x++)
                        {
                            for (int y = 0; y <= MinY - 1; y++)
                            {
                                OutputAnn3.set_Selection(x, y, OutputAnn1.get_Selection(x, y) & OutputAnn2.get_Selection(x, y));
                                
                                OutputAnn4.set_Selection(x, y, OutputAnn1.get_Selection(x, y) || OutputAnn2.get_Selection(x, y));
                                OutputAnn4.set_Selection(x, y, OutputAnn4.get_Selection(x, y) != OutputAnn3.get_Selection(x, y));
                            }
                        }
                        OutputAnn3.ResumeLayout();
                        OutputAnn4.ResumeLayout();
                    }
                }
            }
        }
    }
}
