﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DoctorEye;
using DoctorEye.Plugins;
using System.Drawing;
using System.Windows.Forms;

namespace Forth.Ics.DoctorEye.SDK.Tools.CS
{
    public class SampleTool : Plugin
    {
        // Fields
        private static List<WeakReference> __ENCList = new List<WeakReference>();
        [System.Runtime.CompilerServices.AccessedThroughProperty("MyCustomControl")]
        private SampleControl _MyCustomControl;
        private string _StatusText;

        // Methods
        public SampleTool()
        {
            base.ViewerKeyDown += new EventHandler<KeyEventArgs>(this.SampleTool_ViewerKeyDown);
            base.ViewerKeyUp += new EventHandler<KeyEventArgs>(this.SampleTool_ViewerKeyUp);
            base.ViewerKeyPress += new EventHandler<KeyPressEventArgs>(this.SampleTool_ViewerKeyPress);
            base.ViewerMouseDown += new EventHandler<MouseEventArgs>(this.SampleTool_ViewerMouseDown);
            base.ViewerMouseMove += new EventHandler<MouseEventArgs>(this.SampleTool_ViewerMouseMove);
            base.ViewerMouseUp += new EventHandler<MouseEventArgs>(this.SampleTool_ViewerMouseUp);
            base.ViewerMouseWheel += new EventHandler<MouseEventArgs>(this.SampleTool_ViewerMouseWheel);
            base.ViewerPaint += new EventHandler<PaintEventArgs>(this.SampleTool_ViewerPaint);
            List<WeakReference> list = __ENCList;
            lock (list)
            {
                __ENCList.Add(new WeakReference(this));
            }
            this._StatusText = string.Empty;
            this.MyCustomControl = new SampleControl();
            this.Name = "Sample C# Tool";
            this.Description = "This is a sample Tool created with Visual C#";
            this.Category = "Samples";
            this.CategoryGroup = "C#";
            this.Glyph = global::Forth.Ics.DoctorEye.SDK.Tools.CS.Properties.Resources.ToolImage;
            this.ConfigurationControls = new Control[] { this.MyCustomControl };
        }

        protected override void Activate()
        {
            this.DicomImageViewer.Invalidate();
        }

        protected override void Deactivate()
        {
            this.DicomImageViewer.Invalidate();
        }

        private void MyCustomControl_InvertSelection(object sender, EventArgs e)
        {
            // Create Undo Snapshot.
            this.DicomImage.ActiveAnnotation.CreateUndoSnapshot();

            // Invert all selection map.
            this.DicomImage.ActiveAnnotation.SuspendLayout();
            int num3 = this.DicomImage.Size.Width - 1;
            for (int i = 0; i <= num3; i++)
            {
                int num4 = this.DicomImage.Size.Height - 1;
                for (int j = 0; j <= num4; j++)
                {
                    this.DicomImage.ActiveAnnotation.set_Selection(i, j, !this.DicomImage.ActiveAnnotation.get_Selection(i, j));
                }
            }
            this.DicomImage.ActiveAnnotation.ResumeLayout();
        }

        private void SampleTool_ViewerKeyDown(object sender, KeyEventArgs e)
        {
            this.StatusText = "Event : Key Down => " + e.KeyCode.ToString();
        }

        private void SampleTool_ViewerKeyPress(object sender, KeyPressEventArgs e)
        {
            this.StatusText = "Event : Key Pressed => " + e.KeyChar.ToString();
        }

        private void SampleTool_ViewerKeyUp(object sender, KeyEventArgs e)
        {
            this.StatusText = "Event : Key Up => " + e.KeyCode.ToString();
        }

        private void SampleTool_ViewerMouseDown(object sender, MouseEventArgs e)
        {
            this.StatusText = "Event : Mouse Down => " + e.Button.ToString();
        }

        private void SampleTool_ViewerMouseMove(object sender, MouseEventArgs e)
        {
            this.StatusText = "Event : Mouse Move => " + e.Location.ToString();
        }

        private void SampleTool_ViewerMouseUp(object sender, MouseEventArgs e)
        {
            this.StatusText = "Event : Mouse Up => " + e.Button.ToString();
        }

        private void SampleTool_ViewerMouseWheel(object sender, MouseEventArgs e)
        {
            this.StatusText = "Event : Mouse Wheel => " + e.Delta.ToString();
        }

        private void SampleTool_ViewerPaint(object sender, PaintEventArgs e)
        {
            e.Graphics.DrawString(this.StatusText, this.DicomImageViewer.Font, Brushes.Red, (float)30f, (float)10f);
        }

        // Properties
        private SampleControl MyCustomControl
        {
            get
            {
                return this._MyCustomControl;
            }
            set
            {
                EventHandler<EventArgs> handler = new EventHandler<EventArgs>(this.MyCustomControl_InvertSelection);
                if (this._MyCustomControl != null)
                {
                    this._MyCustomControl.InvertSelection -= handler;
                }
                this._MyCustomControl = value;
                if (this._MyCustomControl != null)
                {
                    this._MyCustomControl.InvertSelection += handler;
                }
            }
        }

        private string StatusText
        {
            get
            {
                return this._StatusText;
            }
            set
            {
                if (this._StatusText != value)
                {
                    this._StatusText = value;
                    this.DicomImageViewer.Invalidate();
                }
            }
        }
    }

}
