﻿Friend Class CreateSessionsControl

    Private _Plugin As Global.DoctorEye.Plugins.Plugin

    Private MyFirstSession As Global.DoctorEye.Session
    Private MyFirstDataSet As Global.DoctorEye.DicomDataSet

    Private MySecondSession As Global.DoctorEye.Session
    Private MySecondDataSet As Global.DoctorEye.DicomDataSet

    Private MyOutputSession As Global.DoctorEye.Session
    Private MyOutputDataSet As Global.DoctorEye.DicomDataSet

    Private Session1Files As String() = New String() {}
    Private Session2Files As String() = New String() {}
    Private OutputFileTemplate As String = String.Empty

    Friend Property Plugin() As Global.DoctorEye.Plugins.Plugin
        Get
            Return _Plugin
        End Get
        Set(ByVal value As Global.DoctorEye.Plugins.Plugin)
            _Plugin = value
        End Set
    End Property

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        With Me.OFD
            If .ShowDialog(Me) = DialogResult.OK Then
                Me.Label1.Text = String.Format("{0} selected file(s)", .FileNames.Length)
                Me.Session1Files = (From item In .FileNames Order By item).ToArray
            End If
        End With
    End Sub

    Private Sub Button2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button2.Click
        With Me.OFD
            If .ShowDialog(Me) = DialogResult.OK Then
                Me.Label2.Text = String.Format("{0} selected file(s)", .FileNames.Length)
                Me.Session2Files = (From item In .FileNames Order By item).ToArray
            End If
        End With
    End Sub

    Private Sub Button4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button4.Click
        With Me.SFD
            If .ShowDialog(Me) = DialogResult.OK Then
                Me.Label3.Text = IO.Path.GetFileName(.FileName)
                Me.OutputFileTemplate = .FileName
            End If
        End With
    End Sub

    Private Sub Button3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button3.Click
        If Me.Session1Files.Length = 0 OrElse Me.Session2Files.Length = 0 OrElse String.IsNullOrEmpty(Me.OutputFileTemplate) Then
            MessageBox.Show("Please fill all the required information and try again", "Incomplete information", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Return
        End If

        Me.CreateSessions()

        Me.ExecuteCalculations()

        ' After all the loading and calculations is finished, show the output session.
        Me.MyOutputSession.Visible = True
    End Sub

    Private Sub CreateSessions()
        ' Create the first session
        MyFirstSession = Global.DoctorEye.Application.AddSession

        'Add a new dataset to the session we created to store our images.
        MyFirstDataSet = MyFirstSession.AddDataset

        Using TheDataSet = Global.DoctorEye.DicomDataSet.CreateNew
            For Each item In Me.Session1Files
                TheDataSet.AddImage(item).Load(False)
            Next

            Dim SortedList = From item In TheDataSet.Images _
                             Order By item.InstanceNumber, item.FullName _
                             Select item.FullName

            ' Add the images and load them
            For Each item In SortedList
                Dim MyFirstImage = MyFirstDataSet.AddImage(item)
                MyFirstImage.Load(True)
            Next

        End Using

        ' After all the loading is done, show the session
        MyFirstSession.Visible = True

        ' Create the second session
        MySecondSession = Global.DoctorEye.Application.AddSession

        'Add a new dataset to the session we created to store our images.
        MySecondDataSet = MySecondSession.AddDataset

        Using TheDataSet = Global.DoctorEye.DicomDataSet.CreateNew
            For Each item In Me.Session2Files
                TheDataSet.AddImage(item).Load(False)
            Next

            Dim SortedList = From item In TheDataSet.Images _
                             Order By item.InstanceNumber, item.FullName _
                             Select item.FullName

            ' Add the images and load them
            For Each item In SortedList
                Dim MySecondImage = MySecondDataSet.AddImage(item)
                MySecondImage.Load(True)
            Next

        End Using

        ' After all the loading is done, show the session
        MySecondSession.Visible = True

        ' Create the output session
        MyOutputSession = Global.DoctorEye.Application.AddSession

        'Add a new dataset to the session we created to store our images.
        MyOutputDataSet = MyOutputSession.AddDataset

        Dim FilePath = IO.Path.GetDirectoryName(Me.OutputFileTemplate)
        Dim FilenameWithoutExt = IO.Path.GetFileNameWithoutExtension(Me.OutputFileTemplate)
        Dim FilenameExt = IO.Path.GetExtension(Me.OutputFileTemplate)

        For i = 0 To Math.Min(Me.Session1Files.Length, Me.Session2Files.Length) - 1

            Dim FirstSize As Size = Me.MyFirstDataSet.Images(i).Size
            Dim SecondSize = Me.MySecondDataSet.Images(i).Size

            ' We keep the maximum size of the two images for the result image
            Dim OutputSize = New Size(Math.Max(FirstSize.Width, SecondSize.Width), Math.Max(FirstSize.Height, SecondSize.Height))

            Dim OutputFilename = IO.Path.Combine(FilePath, IO.Path.ChangeExtension(String.Format("{0}_{1}", FilenameWithoutExt, (i + 1)), FilenameExt))

            If Me.RadioButtonCopyFromFirst.Checked Then
                ' Copy the image to the output directory
                IO.File.Copy(Me.MyFirstDataSet.Images(i).FullName, OutputFilename, True)

                ' Add the Dicom Image to the list
                MyOutputDataSet.AddImage(OutputFilename).Load(False)
            Else
                ' Create the blank Dicom Image and add it to the list
                MyOutputDataSet.AddImage(OutputFilename, OutputSize).Load(True)
            End If
        Next
    End Sub

    Private Sub ExecuteCalculations()

        For i = 0 To Math.Min(Me.MyFirstDataSet.Images.Count, Me.MySecondDataSet.Images.Count) - 1
            Dim MyFirstImage = Me.MyFirstDataSet.Images(i)
            Dim MySecondImage = Me.MySecondDataSet.Images(i)
            Dim MyOutputImage = Me.MyOutputDataSet.Images(i)

            ' We get a list of unique annotation labels.
            Dim AnnotationLabels = (From ann1 In MyFirstImage.Annotations _
                                    Select ann1.Label _
                                    From ann2 In MySecondImage.Annotations _
                                    Select ann2.Label).Distinct

            For Each label In AnnotationLabels
                Dim LabelToSearch = label

                Dim Ann1 = (From item In MyFirstImage.Annotations _
                            Where item.Label = LabelToSearch).FirstOrDefault

                Dim Ann2 = (From item In MySecondImage.Annotations _
                            Where item.Label = LabelToSearch).FirstOrDefault

                ' We need annotations that exist on both images.
                If Ann1 IsNot Nothing AndAlso Ann2 IsNot Nothing Then
                    ' We copy the first annotation to the output image
                    Dim OutputAnn1 = MyOutputImage.AddAnnotation(String.Format("{0} (First Image)", LabelToSearch))
                    OutputAnn1.SuspendLayout()
                    OutputAnn1.Color = Color.FromArgb(50, Color.Red.R, Color.Red.G, Color.Red.B)
                    OutputAnn1.Selection = Ann1.Selection
                    OutputAnn1.ResumeLayout()

                    ' We copy the second annotation to the output image
                    Dim OutputAnn2 = MyOutputImage.AddAnnotation(String.Format("{0} (Second Image)", LabelToSearch))
                    OutputAnn2.SuspendLayout()
                    OutputAnn2.Color = Color.FromArgb(50, Color.Green.R, Color.Green.G, Color.Green.B)
                    OutputAnn2.Selection = Ann2.Selection
                    OutputAnn2.ResumeLayout()

                    ' We create a third annotation with the common selection to the output image
                    Dim OutputAnn3 = MyOutputImage.AddAnnotation(String.Format("{0} (Common)", LabelToSearch))
                    OutputAnn3.SuspendLayout()
                    OutputAnn3.Color = Color.FromArgb(50, Color.Blue.R, Color.Blue.G, Color.Blue.B)

                    ' We create a fourth annotation with the subtraction selection to the output image
                    Dim OutputAnn4 = MyOutputImage.AddAnnotation(String.Format("{0} (Subtraction)", LabelToSearch))
                    OutputAnn4.SuspendLayout()
                    OutputAnn4.Color = Color.FromArgb(50, Color.Cyan.R, Color.Cyan.G, Color.Cyan.B)

                    ' The common selection points must be in the common "areas" of the two images.
                    Dim MinX = Math.Min(MyFirstImage.Size.Width, MySecondImage.Size.Width)
                    Dim MinY = Math.Min(MyFirstImage.Size.Height, MySecondImage.Size.Height)

                    ' We select the common and subtraction points of the two images.
                    For x = 0 To MinX - 1
                        For y = 0 To MinY - 1
                            OutputAnn3.Selection(x, y) = (OutputAnn1.Selection(x, y) And OutputAnn2.Selection(x, y))
                            OutputAnn4.Selection(x, y) = ((OutputAnn2.Selection(x, y) Or OutputAnn1.Selection(x, y)) And Not (OutputAnn3.Selection(x, y)))
                        Next
                    Next

                    OutputAnn3.ResumeLayout()
                    OutputAnn4.ResumeLayout()
                End If

            Next

        Next

    End Sub

End Class
