﻿Imports DoctorEye
Imports DoctorEye.Plugins

Public Class SampleTool
    Inherits Plugin

#Region " Fields "

    Private _StatusText As String = String.Empty
    Private WithEvents MyCustomControl As New SampleControl

#End Region

#Region " Methods "

    Public Sub New()
        Me.Name = "Sample VB Tool"
        Me.Description = "This is a sample Tool created with Visual Basic"
        Me.Category = "Samples"
        Me.CategoryGroup = "Visual Basic"
        Me.Glyph = My.Resources.ToolImage
        Me.ConfigurationControls = New Control() {Me.MyCustomControl}
    End Sub

    Protected Overrides Sub Activate()
        Me.DicomImageViewer.Invalidate()
    End Sub

    Protected Overrides Sub Deactivate()
        Me.DicomImageViewer.Invalidate()
    End Sub

#End Region

#Region " Properties "

    Private Property StatusText() As String
        Get
            Return _StatusText
        End Get
        Set(ByVal value As String)
            If _StatusText = value Then
                Return
            End If
            _StatusText = value
            Me.DicomImageViewer.Invalidate()
        End Set
    End Property

#End Region

#Region " Event Handlers "

    Private Sub MyCustomControl_InvertSelection(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyCustomControl.InvertSelection
        ' Create Undo Snapshot.
        Me.DicomImage.ActiveAnnotation.CreateUndoSnapshot()

        ' Invert all selection map.
        Me.DicomImage.ActiveAnnotation.SuspendLayout()
        For x = 0 To Me.DicomImage.Size.Width - 1
            For y = 0 To Me.DicomImage.Size.Height - 1
                Me.DicomImage.ActiveAnnotation.Selection(x, y) = Not Me.DicomImage.ActiveAnnotation.Selection(x, y)
            Next
        Next
        Me.DicomImage.ActiveAnnotation.ResumeLayout()
    End Sub

    Private Sub SampleTool_ViewerKeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.ViewerKeyDown
        Me.StatusText = "Event : Key Down => " & e.KeyCode.ToString
    End Sub

    Private Sub SampleTool_ViewerKeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles Me.ViewerKeyPress
        Me.StatusText = "Event : Key Pressed => " & e.KeyChar
    End Sub

    Private Sub SampleTool_ViewerKeyUp(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.ViewerKeyUp
        Me.StatusText = "Event : Key Up => " & e.KeyCode.ToString
    End Sub

    Private Sub SampleTool_ViewerMouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.ViewerMouseDown
        Me.StatusText = "Event : Mouse Down => " & e.Button.ToString
    End Sub

    Private Sub SampleTool_ViewerMouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.ViewerMouseMove
        Me.StatusText = "Event : Mouse Move => " & e.Location.ToString
    End Sub

    Private Sub SampleTool_ViewerMouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.ViewerMouseUp
        Me.StatusText = "Event : Mouse Up => " & e.Button.ToString
    End Sub

    Private Sub SampleTool_ViewerMouseWheel(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.ViewerMouseWheel
        Me.StatusText = "Event : Mouse Wheel => " & e.Delta.ToString
    End Sub

    Private Sub SampleTool_ViewerPaint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles Me.ViewerPaint
        e.Graphics.DrawString(Me.StatusText, Me.DicomImageViewer.Font, Brushes.Red, 30, 10)
    End Sub

#End Region

End Class
